"""
I2C Sensor Library

YOOVO Python library for managing I2C sensors with dynamic loading capabilities
and resource management.
"""

from .i2c_bus_manager import I2CBusManager
from .ads_wrapper import ADS1115Wrapper

__version__ = "1.0.0"
__all__ = [
    "I2CBusManager",
    "ADS1115Wrapper",
]
