class _YoovoBypassedValue:
    def __call__(self, *args, **kwargs):
        return None

    def __getattr__(self, name):
        return self

    def __bool__(self):
        return False

    def __int__(self):
        return 0

    def __float__(self):
        return 0.0

    def __str__(self):
        return ""


class _YoovoBypassedSensor:
    _value = _YoovoBypassedValue()

    def __getattr__(self, name):
        return self._value

    def __bool__(self):
        return False


__all__ = ["_YoovoBypassedSensor", "_YoovoBypassedValue"]